/************************************************************************************
The code is designed and produced by MDY Science and Education Co., Ltd, which has the entire ownership. It is only for personal learning, which cannot be used for commercial or profit-making purposes without permission.

    MDY's Mission: Develop Chip Talents and Realize National Chip Dream.

    We sincerely hope that our students can learn the real IC / FPGA code through our standard and rigorous code.

    For more FPGA learning materials, please visit the Forum: http://fpgabbs.com/ and official website: http://www.mdy-edu.com/index.html 

    *************************************************************************************/

module lcd_driver(
    clk          ,//40MHz
    rst_n        ,

    hys          , 
    vys          , 
    lcd_de       ,  
    lcd_rgb      , 
    lcd_dclk   
);

   input                    clk             ;
   input                    rst_n           ;

   output                   hys             ;
   output                   vys             ;
   output                   lcd_de          ;
   output [23:0]            lcd_rgb         ;
   output                   lcd_dclk        ;

   reg                      hys             ;
   reg                      vys             ;
   reg                      lcd_de          ;
   reg    [23:0]            lcd_rgb         ;
   wire                     lcd_dclk        ;

   //1056
   parameter         THPW      = 20         ;   
   parameter         THB       = 46         ;   
   parameter         THD       = 800        ;   
   parameter         THFP      = 210        ;   
   
   //525
   parameter         TVPW      = 10         ;   
   parameter         TVB       = 23         ;   
   parameter         TVD       = 480        ;   
   parameter         TVFP      = 22         ;   

   parameter       HDE_CENTRE  = THD/2      ;//400
   parameter       VDE_CENTRE  = TVD/2      ;//240

   reg   [ 10:0]            h_cnt           ;
   wire                     add_h_cnt       ;
   wire                     end_h_cnt       ;
   reg   [ 9:0]             v_cnt           ;
   wire                     add_v_cnt       ;
   wire                     end_v_cnt       ;


   wire                     active_area     ;
   reg                      rom_area        ;
   reg      [15:0]          rom_addr        ;
   wire     [7:0]           rom_data        ;


always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        h_cnt <= 0; 
    end
    else if(add_h_cnt) begin
        if(end_h_cnt)
            h_cnt <= 0; 
        else
            h_cnt <= h_cnt+1 ;
   end
end
assign add_h_cnt = 1;
assign end_h_cnt = add_h_cnt  && h_cnt == (THB + THD + THFP)-1 ;



always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        v_cnt <= 0; 
    end
    else if(add_v_cnt) begin
        if(end_v_cnt)
            v_cnt <= 0; 
        else
            v_cnt <= v_cnt+1 ;
   end
end
assign add_v_cnt = end_h_cnt;
assign end_v_cnt = add_v_cnt  && v_cnt == (TVB + TVD + TVFP)-1 ;

/*******************************************************/
    //dclk
    assign lcd_dclk = clk;

    //hsync
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            hys <= 0;
        end
        else if(add_h_cnt && h_cnt==THPW-1)begin
            hys <= 1;
        end
        else if(end_h_cnt)begin
            hys <= 0;
        end
    end


    //vsync
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            vys <= 0;
        end
        else if(add_v_cnt && v_cnt==TVPW-1)begin
            vys <= 1;
        end
        else if(end_v_cnt)begin
            vys <= 0;
        end
    end
   

    //lcd_de
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            lcd_de <= 0;
        end
        else if(active_area)begin
            lcd_de <= 1;
        end
        else begin
            lcd_de <= 0;
        end
    end
    

/********************************************************************/    



assign active_area = h_cnt>=(THB-1) && h_cnt<(THB+THD-1) && v_cnt>=(TVB-1) && v_cnt<(TVB+TVD-1);


always  @(*)begin
    rom_area = h_cnt >=((HDE_CENTRE-60) + (THB-1)) && h_cnt < ((HDE_CENTRE+60) + (THB-1)) && v_cnt >= ((VDE_CENTRE-27)+(TVB-1)) && v_cnt < ((VDE_CENTRE+28) +(TVB-1));
end


always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        lcd_rgb <= 0;
    end
    else if(active_area)begin
        if(rom_area)
            lcd_rgb <= {rom_data[7:5],5'b11111,rom_data[4:2],5'b11111,rom_data[1:0],6'b111111};
        else 
            lcd_rgb <= {24{1'b1}};
    end
    else begin
        lcd_rgb <=0;
    end
end


always  @(*)begin
    if(active_area && rom_area)begin
            rom_addr = (h_cnt-((HDE_CENTRE-60)+(THB-1))) + 120*(v_cnt-((VDE_CENTRE-27)+(TVB-1)));
    end
end




fpga_rom u_fpga_rom(
	           .address (rom_addr),
	           .clock   (clk     ),
	           .q       (rom_data));


    endmodule
